@@ -17,27 +17,27 @@
  *************************************************************************************/
 
 #include "parser.h"
+#include "fake.h"
 
 #include <config.h>
 
 #include <QtCore/QFile>
-#include <KDebug>
-
-#include <qjson/parser.h>
-#include <qjson/qobjecthelper.h>
+#include <QLoggingCategory>
+#include <QJsonDocument>
+#include <QJsonArray>
+#include <QJsonObject>
 
 using namespace KScreen;
 
 Config* Parser::fromJson(const QByteArray& data)
 {
     Config *config =  new Config();
-    QJson::Parser parser;
 
-    QVariantMap json = parser.parse(data).toMap();
+    QJsonObject json = QJsonDocument::fromJson(data).object();
 
-    Screen* screen = Parser::screenFromJson(json["screen"].toMap());
+    Screen* screen = Parser::screenFromJson(json["screen"].toObject().toVariantMap());
 
-    QList <QVariant> outputs = json["outputs"].toList();
+    QVariantList outputs = json["outputs"].toArray().toVariantList();
     if (outputs.isEmpty()) {
         return config;
     }
@@ -45,7 +45,7 @@ Config* Parser::fromJson(const QByteArray& data)
     Output *output;
     OutputList outputList;
     Q_FOREACH(const QVariant &value, outputs) {
-        output = Parser::outputFromJson(value);
+        output = Parser::outputFromJson(value.toMap());
         outputList.insert(output->id(), output);
     }
 
@@ -62,20 +62,40 @@ Config* Parser::fromJson(const QString& path)
     return Parser::fromJson(file.readAll());
 }
 
-Screen* Parser::screenFromJson(const QMap< QString, QVariant >& data)
+Screen* Parser::screenFromJson(const QVariantMap &data)
 {
     Screen* screen = new Screen;
     screen->setId(data["id"].toInt());
     screen->setMinSize(Parser::sizeFromJson(data["minSize"].toMap()));
     screen->setMaxSize(Parser::sizeFromJson(data["maxSize"].toMap()));
     screen->setCurrentSize(Parser::sizeFromJson(data["currentSize"].toMap()));
+    screen->setMaxActiveOutputsCount(data["maxActiveOutputsCount"].toInt());
 
     return screen;
 }
 
-Output* Parser::outputFromJson(const QVariant& data)
+void Parser::qvariant2qobject(const QVariantMap& variant, QObject* object)
+{
+    for ( QVariantMap::const_iterator iter = variant.begin(); iter != variant.end(); ++iter )
+    {
+        QVariant property = object->property( iter.key().toLatin1() );
+        Q_ASSERT( property.isValid() );
+        if ( property.isValid() )
+        {
+            QVariant value = iter.value();
+            if ( value.canConvert( property.type() ) )
+            {
+                value.convert( property.type() );
+                object->setProperty( iter.key().toLatin1(), value );
+            } else if ( QString( QLatin1String("QVariant") ).compare( QLatin1String( property.typeName() ) ) == 0) {
+                object->setProperty( iter.key().toLatin1(), value );
+            }
+        }
+    }
+}
+
+Output* Parser::outputFromJson(QMap< QString, QVariant > map)
 {
-    QVariantMap map = data.toMap();
     Output *output = new Output;
     output->setId(map["id"].toInt());
 
@@ -85,8 +105,7 @@ Output* Parser::outputFromJson(const QVariant& data)
         preferredModes.append(mode.toString());
     }
     output->setPreferredModes(preferredModes);
-
-    QJson::QObjectHelper::qvariant2qobject(map, output);
+    map.remove(QLatin1Literal("preferredModes"));
 
     Mode *mode;
     ModeList modelist;
@@ -96,6 +115,7 @@ Output* Parser::outputFromJson(const QVariant& data)
         modelist.insert(mode->id(), mode);
     }
     output->setModes(modelist);
+    map.remove(QLatin1Literal("modes"));
 
     if(map.contains("clones")) {
         QList<int> clones;
@@ -104,6 +124,7 @@ Output* Parser::outputFromJson(const QVariant& data)
         }
 
         output->setClones(clones);
+        map.remove(QLatin1Literal("clones"));
     }
 
     QString type = map["type"].toByteArray().toUpper();
@@ -140,8 +161,14 @@ Output* Parser::outputFromJson(const QVariant& data)
     } else if (type.contains("Unknown")) {
         output->setType(Output::Unknown);
     } else {
-        kDebug() << "Output Type not translated:" << type;
+        qCWarning(KSCREEN_FAKE) << "Output Type not translated:" << type;
     }
+    map.remove(QLatin1Literal("type"));
+
+    //Remove some extra properties that we do not want or need special treatment
+    map.remove(QLatin1Literal("edid"));
+
+    Parser::qvariant2qobject(map, output);
     return output;
 }
 
@@ -149,8 +176,7 @@ Mode* Parser::modeFromJson(const QVariant& data)
 {
     QVariantMap map = data.toMap();
     Mode *mode = new Mode;
-    mode->setId(map["id"].toString());
-    QJson::QObjectHelper::qvariant2qobject(map, mode);
+    Parser::qvariant2qobject(map, mode);
 
     mode->setSize(Parser::sizeFromJson(map["size"].toMap()));
 
