@@ -21,25 +21,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "xlibandxcb.h"
 
 #include <xcb/randr.h>
+#include <QX11Info>
 
-#include <QtGui/QX11Info>
-
-static xcb_connection_t *XRandR11XCBConnection = 0;
+Display* display()
+{
+    return QX11Info::display();
+}
 
 xcb_connection_t *connection()
 {
-    // Use our own connection to make sure that we won't mess up Qt's connection
-    // if something goes wrong on our side.
-    if (XRandR11XCBConnection == 0) {
-        XRandR11XCBConnection = xcb_connect(0, 0);
-    }
-    return XRandR11XCBConnection;
-}
-
-void closeConnection()
-{
-    xcb_disconnect(XRandR11XCBConnection);
-    XRandR11XCBConnection = 0;
+    return QX11Info::connection();
 }
 
 xcb_screen_t *screen_of_display (xcb_connection_t *c, int screen)
